#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Testing base functionalities of berhoel.odf library.
"""

# Standard libraries.
from pathlib import Path

# Third party libraries.
import toml
import pytest

from berhoel.odf import Odf, __version__

__date__ = "2020/05/26 22:35:33 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


@pytest.fixture
def py_project(base_path):
    "Return path of project `pyproject.toml`."
    return base_path / "pyproject.toml"


@pytest.fixture
def toml_inst(py_project):
    "Return `toml` instance of project `pyproject.toml`"
    return toml.load(py_project.open("r"))


def test_version(toml_inst):
    "Test for consistent version numbers."
    assert __version__ == toml_inst["tool"]["poetry"]["version"]


def test_init(data1):
    _ = Odf(data1)


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Processing ods (OpenDocument spreadsheets).
"""

# Standard libraries.
import re
import typing
import datetime
import functools

from . import Odf, OdfXml

__date__ = "2020/06/07 18:41:50 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"

# Identify hyperlink in formula.
IMDB_HYPERLINK = re.compile(
    r"of:=HYPERLINK\(\s*"
    r'"(?P<url>https?://www.imdb.com/title/tt[0-9]+)'
    r'(?:/?(?:[?]ref_=(?:[a-z]{2}_)+(?:[0-9]{1,3}|[a-z]{1,3})|(?:episodes))?)?"\s*;\s*'
    r'"(?P<name>.+)"\s*\)'
)

# Identify relations to other cells in formulas.
LINKED_CELL = re.compile(
    r"of:=-(:?[-+]?[0-9]*\.?[0-9]*[-+*/])*\[\$?(?P<sheet>'k\&v')"
    r"\.(?P<row>F)(?P<line>[0-9]+)]"
)


class P(OdfXml):
    """Representing a paragraph in a cell."""

    @property
    def text(self) -> typing.Optional[str]:
        """Get text from paragraph.

Returns:
   str: Whole paragraph text if available.
"""
        res = " ".join([i for i in self.root.itertext()])
        return res if res.strip() else None


class Cell(OdfXml):
    """Representing a cell in a table row."""

    @property
    def text(self) -> typing.Optional[str]:
        """Return text associated with cell.

Returns:
   str: Cell text if avaliable.
"""
        if self.root is None:
            return None
        res = self.find("text:p")
        return None if res is None else P(res).text

    @property
    def date(self) -> typing.Optional[datetime.date]:
        """Return date value of cell if available.

Returns:
   datetime.date: Date value of cell if available.
"""
        val = self.get("office:date-value")
        return (
            None if val is None else datetime.datetime.strptime(val, "%Y-%m-%d").date()
        )

    @property
    def value(self) -> typing.Optional[str]:
        """Return value of cell.

Returns:
   str: Value of cell if available.
"""
        return self.get("office:value")

    @property
    def float(self) -> typing.Optional[float]:
        """Return float value of cell.

Returns:
   float: ``float`` falue of cell if available.
"""
        return None if self.value is None else float(self.value)

    @property
    def int(self) -> typing.Optional[int]:
        """Return integer value of cell.

Returns:
   int: ``int`` value if cell if available.
"""
        return None if self.value is None else int(self.value)

    @property
    def url(self) -> typing.Optional[str]:
        """Return URL associated with cell from hyperlink function.

Returns:
   str: url from ``HYPERLINK`` formula or cell hyperlink.
"""
        s_val: str = self.get("table:formula")
        val = None if s_val is None else IMDB_HYPERLINK.match(s_val)
        return None if val is None else val.group("url")

    @property
    def link(self) -> typing.Optional[typing.Dict[str, str]]:
        """Provide Link information.

This link information is information of values referenced from diffrent tables in
formula.

Returns:
   dict: Result of ``re.match.groupdict``. Provided keys are ``sheet``, ``row``, and ``line.``
"""
        dvd_link: str = self.get("table:formula")
        if dvd_link is not None:
            match = LINKED_CELL.match(dvd_link)
            if match is not None:
                return match.groupdict()
        return None


class Row(OdfXml):
    """Representing table row.
"""

    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        self.content_cell: str = self._attrib_map("table:table-cell")
        self.covered_cell: str = self._attrib_map("table:covered-table-cell")
        self.columns_repeated: str = self._attrib_map("table:number-columns-repeated")

    @functools.cached_property
    def cells(self) -> typing.List[typing.Optional[Cell]]:
        """Provide list of cells in row.

Empty cells are replaced by ``None``, as are cells covered by merged cells.

Returns:
   List[Cell]: The cells making up the current row.
"""
        res: typing.List[typing.Optional[Cell]] = []
        for cell in self.root:
            if cell.tag == self.content_cell:
                elem = Cell(cell)
                res.extend(
                    [elem]
                    * (
                        int(elem.get("table:number-columns-repeated"))
                        if self.columns_repeated in cell.attrib
                        else 1
                    )
                )
            elif cell.tag == self.covered_cell:
                elem = Cell(cell)
                res.extend(
                    [None]
                    * (
                        int(elem.get("table:number-columns-repeated"))
                        if self.columns_repeated in cell.attrib
                        else 1
                    )
                )
            else:
                raise NotImplementedError(f"Unknown tag {cell.tag}")
        return res


class Table(OdfXml):
    """Representation of spreadsheet table.
"""

    @functools.cached_property
    def name(self) -> str:
        """Return name attriute of table.

Returns:
   str: table name.
"""
        return self.get("table:name")

    @functools.cached_property
    def rows(self) -> typing.List[Row]:
        """Return rows in table.

Returns:
   List[Row]: list of ``Row`` intances.
"""
        return [Row(e) for e in self.findall("table:table-row")]

    @functools.cached_property
    def style_name(self) -> str:
        """Return table style-name.

I Know of ``ta1`` for ordinary tables, and ``ta2`` for hidden tables.

Returns:
   str: style name of table.
"""
        return self.get("table:style-name")

    @property
    def hidden(self) -> bool:
        """Return whether table is hidden.

Returns:
   bool: ``True`` if table is hidden.
"""
        return self.style_name == "ta2"


class Ods(Odf):
    """Processing ODS files.
"""

    def __init__(self, *args, **kw):
        """Open ODF spreadsheet file.
"""
        super().__init__(*args, **kw)

        self.root = self.find("office:body/office:spreadsheet")

    @functools.cached_property
    def tables(self) -> typing.List[Table]:
        """Return all tables in spreadsheet.

Returns:
   List[Table]: Tables in OpenOffice spreadsheet.
"""
        return [Table(e) for e in self.findall("table:table")]


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Testing accessing ods files.
"""

# Third party libraries.
import pytest

from berhoel.odf.ods import LINKED_CELL, IMDB_HYPERLINK, Ods

__date__ = "2020/05/26 21:49:34 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


@pytest.fixture
def ods_sample1(data1):
    return Ods(data1)


def test_tables(ods_sample1):
    assert ods_sample1.tables is not None
    assert len(ods_sample1.tables) == 2


@pytest.fixture
def tables(ods_sample1):
    return ods_sample1.tables


def test_table_names(tables):
    assert tables[0].name == "Tabelle1"
    assert tables[1].name == "Tabelle2"


def test_table_rows(tables):
    assert len(tables[0].rows) == 9
    assert len(tables[1].rows) == 4


def test_table_hidden(tables):
    assert not tables[0].hidden
    assert tables[1].hidden


@pytest.fixture
def rows0(tables):
    return tables[0].rows


@pytest.fixture
def rows1(tables):
    return tables[1].rows


def test_rows0_1(rows0):
    for i, row in enumerate(rows0):
        assert len(row.cells) == 4, f"row: {i}"


def test_rows1_1(rows1):
    for i, row in enumerate(rows1):
        assert len(row.cells) == 3, f"row: {i}"


@pytest.mark.parametrize(
    "test_input, url_ref, title_ref",
    [
        [
            'of:=HYPERLINK("http://www.imdb.com/title/tt0094612/";"Action Jackson")',
            "http://www.imdb.com/title/tt0094612",
            "Action Jackson",
        ],
        [
            'of:=HYPERLINK( "http://www.imdb.com/title/tt0094612/" ; "Action Jackson")',
            "http://www.imdb.com/title/tt0094612",
            "Action Jackson",
        ],
        [
            'of:=HYPERLINK("http://www.imdb.com/title/tt3375370";"The Gunfighter (2014)" )',
            "http://www.imdb.com/title/tt3375370",
            "The Gunfighter (2014)",
        ],
        [
            'of:=HYPERLINK("https://www.imdb.com/title/tt0055205/?ref_=nv_sr_1";"16 Uhr 50 ab Paddington (1961)")',
            "https://www.imdb.com/title/tt0055205",
            "16 Uhr 50 ab Paddington (1961)",
        ],
        [
            'of:=HYPERLINK("https://www.imdb.com/title/tt0043265/?ref_=fn_al_tt_2";"African Queen (1951)")',
            "https://www.imdb.com/title/tt0043265",
            "African Queen (1951)",
        ],
        [
            'of:=HYPERLINK("https://www.imdb.com/title/tt1299368/?ref_=tt_ov_inf";"Southland")',
            "https://www.imdb.com/title/tt1299368",
            "Southland",
        ],
        [
            'of:=HYPERLINK("http://www.imdb.com/title/tt2621446?ref_=tt_ov_inf";"Star Trek: Renegades")',
            "http://www.imdb.com/title/tt2621446",
            "Star Trek: Renegades",
        ],
        [
            'of:=HYPERLINK("http://www.imdb.com/title/tt0284718/episodes";"Crossing Jordan")',
            "http://www.imdb.com/title/tt0284718",
            "Crossing Jordan",
        ],
    ],
)
def test_indb_hyperlink(test_input, url_ref, title_ref):
    match = IMDB_HYPERLINK.match(test_input)
    title, url = match.group("name"), match.group("url")
    assert url == url_ref
    assert title == title_ref


@pytest.mark.parametrize(
    "test_input, val_ref",
    [
        ["of:=-['k&v'.F207]", "207"],
        ["of:=-[$'k&v'.F207]", "207"],
        ["of:=-0.5*['k&v'.F818]", "818"],
    ],
)
def test_dvd_link(test_input, val_ref):
    match = LINKED_CELL.match(test_input)
    val = match.group("line")
    assert val == val_ref


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Give access to ods files.
"""

# Standard libraries.
from typing import List, Union
from pathlib import Path
from zipfile import ZipFile

# Third party libraries.
from lxml import etree

__date__ = "2020/06/07 18:29:20 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"

try:
    from ._version import __version__
except ImportError:
    __version__ = "0.0.0+unknown0"


class OdfXml:
    """Base XML handling class for ODF processing.
"""
    def __init__(self, elem: etree._Element):
        """
Args:
    element (etree._Element): XML Element
        """
        self.root = elem
        self.nsmap = (
            self.root.nsmap if hasattr(elem, "nsmap") else self.root.getroot().nsmap
        )

    def find(self, tag: str) -> etree._Element:
        """Find ``tag`` in this element.

Args:
    tag (str): XML tag to serarch for.

Returns:
   etree._Element: Element acc. to ``tag``.
"""
        return self.root.find(tag, namespaces=self.nsmap)

    def findall(self, tag: str) -> List[etree._Element]:
        """Find all of ``tag``.

Args:
   tag (str): XML tag to find.

Returns:
   List[etree._Element]: Element acc. to ``tag``.
"""
        return self.root.findall(tag, namespaces=self.nsmap)

    def _attrib_map(self, attrib: str) -> str:
        """Helper for ``get``: provide namespace.

Args:
   attrib (str): attribute name in the form of <namespace name>:<attr name>.

Returns:
   str: attribute name with extended namespace.
"""
        ns, tag = attrib.split(":")
        return f"{{{self.nsmap[ns]}}}{tag}"

    def get(self, attrib: str) -> str:
        """Get atribute of this element, honors namespace.

Args:
   attrib(str): Attrribute name.

Returns:
   str: Attribute value.
"""
        return self.root.get(self._attrib_map(attrib))


class Odf(OdfXml):
    """Base class for OpenDocument Format files.
"""

    def __init__(self, path: Union[str, Path]):
        """Open ODF file from ``path``.

Args:
   path ([str,Path]): Location of OpenOffice file.
        """
        with ZipFile(path) as odf_zip:
            with odf_zip.open("content.xml") as content:
                doc = etree.parse(content)
        super().__init__(doc)


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Common settiongs for testing `berhoel.odf`.
"""
# Standard libraries.
from pathlib import Path

# Third party libraries.
import pytest

__date__ = "2020/05/26 22:36:47 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


@pytest.fixture
def base_path():
    "Return path to project base."
    return Path(__file__).parents[3]


@pytest.fixture
def data_path(base_path):
    "Return path to data directory."
    return base_path / "data"


@pytest.fixture
def data1(data_path):
    "Return path to first sample data file."
    return data_path / "sample1.ods"


# Local Variables:
# mode: python
# compile-command: "poetry run tox"
# time-stamp-pattern: "30/__date__ = \"%:y/%02m/%02d %02H:%02M:%02S %u\""
# End:



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['berhoel', 'berhoel.odf', 'berhoel.odf.test']

package_data = \
{'': ['*']}

install_requires = \
['lxml>=4.5.0,<5.0.0', 'pdoc3>=0.8.1,<0.9.0', 'toml>=0.10.0,<0.11.0']

setup_kwargs = {
    'name': 'berhoelodf',
    'version': '0.2.2',
    'description': 'Lightweight and limited access to odf files using lxml.',
    'long_description': '``berhoelODF``\n==============\n\nLightweight and limited access to odf files using lxml.\n\nIt is just used by me for a limited project of converting a\nspreadsheet to ``Django`` data. For me this package works much faster\nthan ``odfpy``. Just loading my 5.7MB ods file took 1.5 minutes. This\npackage is based on ``lxml``, and ``lxml`` parses the XML in 2\nseconds, the whole processing to ``Django`` takes about 1.5 minutes.\n\nIf you have other requirements, the code should be easily extensible.\n\nInstallation\n------------\n\n``pip install berhoelodf``\n\nAvailability\n~~~~~~~~~~~~\n\nThe latest version should be available at my `GitLab\n<https://gitlab.com/berhoel/python/berhoelODF.git>`_ repository, the\npackage is avaliable at `pypi <https://pypi.org/project/berhoelODF/>`_\nvia ``pip install berhoelodf``.\n\nDocumentation\n~~~~~~~~~~~~~\n\nDocumentation is hosted on `höllmanns.de <https://www.höllmanns.de/python/berhoelODF/>`_.\n',
    'author': 'Berthold Höllmann',
    'author_email': 'berthold-gitlab@xn--hllmanns-n4a.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://python.xn--hllmanns-n4a.de/berhoelodf/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
